/*
 * jkeytool - SwingInterface.java -
 * Copyright © 2009 David Roden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package net.pterodactylus.jkeytool.gui.swing;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.io.File;
import java.security.KeyStore;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import net.pterodactylus.jkeytool.core.Core;
import net.pterodactylus.jkeytool.gui.Interface;
import net.pterodactylus.jkeytool.main.Main;
import net.pterodactylus.util.i18n.gui.I18nAction;
import net.pterodactylus.util.i18n.gui.I18nMenu;
import net.pterodactylus.util.swing.StatusBar;

/**
 * TODO
 *
 * @author David Roden &lt;droden@gmail.com&gt;
 */
public class SwingInterface implements Interface {

	/** The core to control. */
	private Core core;

	/** The main frame. */
	private JFrame mainFrame = new JFrame("jkeytool " + Main.getVersion());

	/** The tab pane. */
	private JTabbedPane tabPane = new JTabbedPane();

	/** The status bar. */
	private StatusBar statusBar = new StatusBar();

	/** The “create key store” action. */
	private Action createKeyStoreAction;

	/** The “quit” action. */
	private Action quitAction;

	/** Loaded key stores and their panels. */
	private final Map<KeyStore, KeyStorePanel> keyStores = new HashMap<KeyStore, KeyStorePanel>();

	public SwingInterface() {
		createActions();
		createFrame();
	}

	//
	// ACTIONS
	//

	private void createKeyStore() {
	}

	private void quit() {
		System.exit(0);
	}

	//
	// PRIVATE METHODS
	//

	private void createActions() {
		createKeyStoreAction = new I18nAction("jkeytool.action.createKeyStore") {

			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent actionEvent) {
				createKeyStore();
			}
		};
		quitAction = new I18nAction("jkeytool.action.quit") {

			/**
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			@SuppressWarnings("synthetic-access")
			public void actionPerformed(ActionEvent actionEvent) {
				quit();
			}
		};
	}

	private void createFrame() {
		mainFrame.setJMenuBar(createMenubar());
		mainFrame.getContentPane().add(createCenterPanel(), BorderLayout.CENTER);
		mainFrame.getContentPane().add(statusBar, BorderLayout.PAGE_END);
		mainFrame.pack();
	}

	private JComponent createCenterPanel() {
		JPanel centerPanel = new JPanel(new BorderLayout());
		centerPanel.setBorder(BorderFactory.createEmptyBorder(6, 6, 6, 6));

		centerPanel.add(tabPane, BorderLayout.CENTER);

		return centerPanel;
	}

	private JMenuBar createMenubar() {
		JMenuBar menubar = new JMenuBar();

		JMenu fileMenu = new I18nMenu("jkeytool.menu.file");
		menubar.add(fileMenu);
		fileMenu.add(createKeyStoreAction);
		fileMenu.addSeparator();
		fileMenu.add(quitAction);

		return menubar;
	}

	//
	// INTERFACE Interface
	//

	/**
	 * Sets the core to control.
	 *
	 * @param core
	 *            The core to control
	 */
	public void setCore(Core core) {
		this.core = core;
	}

	/**
	 * {@inheritDoc}
	 */
	public void start() {
		mainFrame.setVisible(true);
		statusBar.setText("jkeytool startup complete.");
		core.loadKeyStore(new File("client.p12"));
	}

	/**
	 * {@inheritDoc}
	 */
	public void stop() {
		/* TODO */
	}

	//
	// INTERFACE CoreListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void keyStoreCreated(KeyStore keyStore) {
		/* TODO */
	}

	/**
	 * {@inheritDoc}
	 */
	public void keyStoreLoaded(File keyStoreFile, KeyStore keyStore) {
		KeyStorePanel keyStorePanel = new KeyStorePanel(keyStore);
		keyStores.put(keyStore, keyStorePanel);
		tabPane.addTab(keyStoreFile.getName(), keyStorePanel);
	}

	/**
	 * {@inheritDoc}
	 */
	public void keyStoreNotCreated(String keyStoreType, Throwable reason) {
		/* TODO */
	}

	/**
	 * {@inheritDoc}
	 */
	public void keyStoreNotLoaded(File keyStoreFile) {
		/* TODO */
	}

}
